@@ -585,11 +585,9 @@ static void
 emp_uri_popup_link_open(EPopup *ep, EPopupItem *item, void *data)
 {
 	EMPopupTargetURI *t = (EMPopupTargetURI *)ep->target;
-	gchar *unescaped_uri = em_utils_url_unescape_amp (t->uri);
 
 	/* FIXME Pass a parent window. */
-	e_show_uri (NULL, unescaped_uri);
-	g_free (unescaped_uri);
+	e_show_uri (NULL, t->uri);
 }
 
 static void
@@ -799,13 +797,14 @@ emp_standard_menu_factory(EPopup *emp, void *data)
 		apps = g_app_info_get_all_for_type (cp ? cp : mime_type);
 		g_free (cp);
 
-		if (apps == NULL && strcmp (mime_type, "application/octet-stream") == 0) {
+		if (apps == NULL || strcmp (mime_type, "application/octet-stream") == 0) {
 			if (filename != NULL) {
 				gchar *name_type;
 
 				name_type = e_util_guess_mime_type (filename, FALSE);
 				cp = g_content_type_from_mime_type (name_type);
-				apps = g_app_info_get_all_for_type (cp ? cp : name_type);
+				/* show alternative apps first */
+				apps = g_list_concat (g_app_info_get_all_for_type (cp ? cp : name_type), apps);
 				g_free (cp);
 				g_free (name_type);
 			}
