@@ -1,16 +1,31 @@
 /*
-   Copyright (C) 2004 Novell, Inc.
-   Author: Radek Doulik
-
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) version 3.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with the program; if not, see <http://www.gnu.org/licenses/>  
+ *
+ *
+ * Authors:
+ *		Radek Doulik <rodo@ximian.com>
+ *
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
+ *
  */
 
-/* This file is licensed under the GNU GPL v2 or later */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include "e-util/e-icon-factory.h"
+#include <gtk/gtk.h>
+#include <glib/gstdio.h>
 #include "e-util/e-mktemp.h"
 #include "camel/camel-medium.h"
 #include "camel/camel-mime-part.h"
@@ -18,11 +33,6 @@
 #include "camel/camel-stream-fs.h"
 #include "mail/em-format-hook.h"
 #include "mail/em-format-html.h"
-#include "glib/gstdio.h"
-#include "gtk/gtkbutton.h"
-#include "gtk/gtkbox.h"
-#include "gtk/gtkimage.h"
-#include "gtk/gtkhbbox.h"
 #include "gtkhtml/gtkhtml-embedded.h"
 #include "gst/gst.h"
 
@@ -253,7 +263,7 @@ org_gnome_audio_inline_add_button (GtkWidget *box, const char *stock_icon, GCall
 	g_signal_connect (button, "clicked", cb, data);
 
 	gtk_widget_show (button);
-	gtk_box_pack_end_defaults (GTK_BOX (box), button);
+	gtk_box_pack_end (GTK_BOX (box), button, TRUE, TRUE, 0);
 
 	return button;
 }
